@@ -1,28 +1,28 @@
 """
 """
 
-from enum import Enum
-from .utils import get_parameter_args
-from .indexes import to_xmlid, AMDADatasetIndex, AMDAParameterIndex, AMDATimetableIndex, AMDACatalogIndex, \
-    AMDAComponentIndex, AMDAIndex
-
-from ._impl import is_public, is_private
-
-from datetime import datetime
-from typing import Optional, List, Dict, Union
-
-# General modules
-from speasy.config import amda_user_cache_retention
-from speasy.core.cache import Cacheable, CacheCall, CACHE_ALLOWED_KWARGS
-from speasy.core.datetime_range import DateTimeRange
-from ...core import make_utc_datetime, AllowedKwargs
-from speasy.products.variable import SpeasyVariable
-from speasy.products.dataset import Dataset
-from speasy.products.timetable import TimeTable
-from speasy.products.catalog import Catalog
-from speasy.core.proxy import Proxyfiable, GetProduct, PROXY_ALLOWED_KWARGS
-from speasy.inventory import flat_inventories
 import logging
+from datetime import datetime
+from enum import Enum
+from typing import Dict, List, Optional, Union
+
+from ...config import amda as amda_cfg
+from ...core import AllowedKwargs, make_utc_datetime
+from ...core.cache import CACHE_ALLOWED_KWARGS, Cacheable, CacheCall
+from ...core.dataprovider import (GET_DATA_ALLOWED_KWARGS, DataProvider,
+                                  ParameterRangeCheck)
+from ...core.datetime_range import DateTimeRange
+from ...core.inventory.indexes import (CatalogIndex, ComponentIndex,
+                                       DatasetIndex, ParameterIndex,
+                                       SpeasyIndex, TimetableIndex)
+from ...core.proxy import PROXY_ALLOWED_KWARGS, GetProduct, Proxyfiable
+from ...products.catalog import Catalog
+from ...products.dataset import Dataset
+from ...products.timetable import TimeTable
+from ...products.variable import SpeasyVariable
+from ._impl import is_private, is_public
+from .inventory import to_xmlid
+from .utils import get_parameter_args
 
 log = logging.getLogger(__name__)
 
@@ -38,26 +38,14 @@ class ProductType(Enum):
     CATALOG = 5
 
 
-def _is_user_prod(product_id: str or AMDAIndex, collection: Dict):
+def _is_user_prod(product_id: str or SpeasyIndex, collection: Dict):
     xmlid = to_xmlid(product_id)
     if xmlid in collection:
         return not collection[xmlid].is_public
     return False
 
 
-def is_user_catalog(catalog_id: str or AMDACatalogIndex):
-    return _is_user_prod(catalog_id, flat_inventories.amda.catalogs)
-
-
-def is_user_timetable(timetable_id: str or AMDATimetableIndex):
-    return _is_user_prod(timetable_id, flat_inventories.amda.timetables)
-
-
-def is_user_parameter(parameter_id: str or AMDAParameterIndex):
-    return _is_user_prod(parameter_id, flat_inventories.amda.parameters)
-
-
-class AMDA_Webservice:
+class AMDA_Webservice(DataProvider):
     __datetime_format__ = "%Y-%m-%dT%H:%M:%S.%f"
     """AMDA_Webservice connexion class. This class manages the connexion to AMDA_Webservice. Use the :meth:`get_data` or
     :meth:`get_parameter` methods for retrieving data.
@@ -93,11 +81,27 @@ class AMDA_Webservice:
     def __init__(self, server_url: str = "http://amda.irap.omp.eu"):
         from ._impl import AmdaImpl
         self._impl = AmdaImpl(server_url=server_url)
+        DataProvider.__init__(self, provider_name='amda')
 
     def __del__(self):
         pass
 
-    def product_version(self, parameter_id: str or AMDAParameterIndex):
+    def build_inventory(self, root: SpeasyIndex):
+        return self._impl.build_inventory(root)
+
+    def build_private_inventory(self, root: SpeasyIndex):
+        return self._impl.build_private_inventory(root)
+
+    def is_user_catalog(self, catalog_id: str or CatalogIndex):
+        return _is_user_prod(catalog_id, self.flat_inventory.catalogs)
+
+    def is_user_timetable(self, timetable_id: str or TimetableIndex):
+        return _is_user_prod(timetable_id, self.flat_inventory.timetables)
+
+    def is_user_parameter(self, parameter_id: str or ParameterIndex):
+        return _is_user_prod(parameter_id, self.flat_inventory.parameters)
+
+    def product_version(self, parameter_id: str or ParameterIndex):
         """Get date of last modification of dataset or parameter.
 
         Parameters
@@ -111,7 +115,50 @@ class AMDA_Webservice:
             product version
         """
         dataset = self._find_parent_dataset(parameter_id)
-        return flat_inventories.amda.datasets[dataset].lastUpdate
+        return self.flat_inventory.datasets[dataset].lastUpdate
+
+    def parameter_range(self, parameter_id: str or ParameterIndex) -> Optional[DateTimeRange]:
+        """Get product time range.
+        
+        Parameters
+        ----------
+        parameter_id: str or ParameterIndex
+            parameter id
+            
+        Returns
+        -------
+        Optional[DateTimeRange]
+            Data time range
+            
+        Examples
+        --------
+        >>> import speasy as spz
+        >>> spz.amda.parameter_range("imf")
+        <DateTimeRange: 1997-09-02T00:00:12+00:00 -> ...>
+        """
+        return self._parameter_range(parameter_id)
+
+    def dataset_range(self, dataset_id: str or DatasetIndex) -> Optional[DateTimeRange]:
+        """Get product time range.
+        
+        Parameters
+        ----------
+        dataset_id: str or DatasetIndex
+            parameter id
+            
+        Returns
+        -------
+        Optional[DateTimeRange]
+            Data time range
+            
+        Examples
+        --------
+        >>> import speasy as spz
+        >>> spz.amda.dataset_range("ace-imf-all")
+        <DateTimeRange: 1997-09-02T00:00:12+00:00 -> ...>
+        """
+
+        return self._dataset_range(dataset_id)
 
     def get_data(self, product, start_time=None, stop_time=None, **kwargs) -> Optional[Union[
         SpeasyVariable, TimeTable, Catalog, Dataset]]:
@@ -138,7 +185,7 @@ class AMDA_Webservice:
         >>> imf_data = spz.amda.get_data("imf", "2019-02-24T19:20:05", "2019-02-25")
         >>> print(imf_data.columns)
         ['imf[0]', 'imf[1]', 'imf[2]']
-        >>> print(imf_data.data.shape)
+        >>> print(imf_data.values.shape)
         (1050, 3)
 
 
@@ -147,23 +194,24 @@ class AMDA_Webservice:
         if product_t == ProductType.DATASET and start_time and stop_time:
             return self.get_dataset(dataset_id=product, start=start_time, stop=stop_time, **kwargs)
         if product_t == ProductType.PARAMETER and start_time and stop_time:
-            if is_user_parameter(product):
+            if self.is_user_parameter(product):
                 return self.get_user_parameter(parameter_id=product, start_time=start_time, stop_time=stop_time,
                                                **kwargs)
             else:
                 return self.get_parameter(product=product, start_time=start_time, stop_time=stop_time, **kwargs)
         if product_t == ProductType.CATALOG:
-            if is_user_catalog(product):
+            if self.is_user_catalog(product):
                 return self.get_user_catalog(catalog_id=product, **kwargs)
             else:
                 return self.get_catalog(catalog_id=product, **kwargs)
         if product_t == ProductType.TIMETABLE:
-            if is_user_timetable(product):
+            if self.is_user_timetable(product):
                 return self.get_user_timetable(timetable_id=product, **kwargs)
             else:
                 return self.get_timetable(timetable_id=product, **kwargs)
+        raise ValueError(f"Unknown product: {product}")
 
-    def get_user_parameter(self, parameter_id: str or AMDAParameterIndex, start_time: datetime or str,
+    def get_user_parameter(self, parameter_id: str or ParameterIndex, start_time: datetime or str,
                            stop_time: datetime or str) -> Optional[SpeasyVariable]:
         """Get user parameter. Raises an exception if user is not authenticated.
 
@@ -188,7 +236,7 @@ class AMDA_Webservice:
         >>> user_param = spz.amda.get_user_parameter("ws_0", "2019-02-24T19:20:05", "2019-02-25") # doctest: +SKIP
         >>> print(user_param.columns) # doctest: +SKIP
         ['ws_test_param']
-        >>> print(user_param.data.shape) # doctest: +SKIP
+        >>> print(user_param.values.shape) # doctest: +SKIP
         (2, 1)
 
 
@@ -203,8 +251,8 @@ class AMDA_Webservice:
         start_time, stop_time = make_utc_datetime(start_time), make_utc_datetime(stop_time)
         return self._impl.dl_user_parameter(parameter_id=parameter_id, start_time=start_time, stop_time=stop_time)
 
-    @CacheCall(cache_retention=float(amda_user_cache_retention.get()))
-    def get_user_timetable(self, timetable_id: str or AMDATimetableIndex) -> Optional[TimeTable]:
+    @CacheCall(cache_retention=amda_cfg.user_cache_retention())
+    def get_user_timetable(self, timetable_id: str or TimetableIndex) -> Optional[TimeTable]:
         """Get user timetable. Raises an exception if user is not authenticated.
 
         Parameters
@@ -233,8 +281,8 @@ class AMDA_Webservice:
         timetable_id = to_xmlid(timetable_id)
         return self._impl.dl_user_timetable(timetable_id=timetable_id)
 
-    @CacheCall(cache_retention=float(amda_user_cache_retention.get()))
-    def get_user_catalog(self, catalog_id: str or AMDACatalogIndex) -> Optional[Catalog]:
+    @CacheCall(cache_retention=amda_cfg.user_cache_retention())
+    def get_user_catalog(self, catalog_id: str or CatalogIndex) -> Optional[Catalog]:
         """Get user catalog. Raises an exception if user is not authenticated.
 
 
@@ -265,10 +313,13 @@ class AMDA_Webservice:
         catalog_id = to_xmlid(catalog_id)
         return self._impl.dl_user_catalog(catalog_id=catalog_id)
 
-    @AllowedKwargs(PROXY_ALLOWED_KWARGS + CACHE_ALLOWED_KWARGS + ['product', 'start_time', 'stop_time'])
+    @AllowedKwargs(PROXY_ALLOWED_KWARGS + CACHE_ALLOWED_KWARGS + GET_DATA_ALLOWED_KWARGS)
+    @ParameterRangeCheck()
     @Cacheable(prefix="amda", version=product_version, fragment_hours=lambda x: 12)
     @Proxyfiable(GetProduct, get_parameter_args)
-    def get_parameter(self, product, start_time, stop_time) -> Optional[SpeasyVariable]:
+    def get_parameter(self, product, start_time, stop_time, extra_http_headers: Dict or None = None, **kwargs) -> \
+    Optional[
+        SpeasyVariable]:
         """Get parameter data.
 
         Parameters
@@ -279,8 +330,8 @@ class AMDA_Webservice:
             desired data start time
         stop_time:
             desired data stop time
-        kwargs: dict
-            optional arguments
+        extra_http_headers: dict
+            reserved for internal use
 
         Returns
         -------
@@ -292,19 +343,19 @@ class AMDA_Webservice:
 
         >>> import speasy as spz
         >>> import datetime
-        >>> imf_data = spz.amda.get_parameter("imf", datetime.datetime(2000,1,1), datetime.datetime(2000,1,2))
+        >>> imf_data = spz.amda.get_parameter("imf", "2018-01-01", "2018-01-01T01")
         >>> print(imf_data.columns)
         ['imf[0]', 'imf[1]', 'imf[2]']
-        >>> print(imf_data.data.shape)
-        (5400, 3)
+        >>> print(imf_data.values.shape)
+        (225, 3)
 
         """
-
         log.debug(f'Get data: product = {product}, data start time = {start_time}, data stop time = {stop_time}')
-        return self._impl.dl_parameter(start_time=start_time, stop_time=stop_time, parameter_id=product)
+        return self._impl.dl_parameter(start_time=start_time, stop_time=stop_time, parameter_id=product,
+                                       extra_http_headers=extra_http_headers)
 
-    def get_dataset(self, dataset_id: str or AMDADatasetIndex, start: str or datetime, stop: str or datetime,
-                    **kwargs) -> Dataset:
+    def get_dataset(self, dataset_id: str or DatasetIndex, start: str or datetime, stop: str or datetime,
+                    **kwargs) -> Dataset or None:
         """Get dataset contents. Returns list of SpeasyVariable objects, one for each
         parameter in the dataset.
 
@@ -319,8 +370,8 @@ class AMDA_Webservice:
 
         Returns
         -------
-        Dataset
-            dataset content as a collection of SpeasyVariable
+        Dataset or None
+            dataset content as a collection of SpeasyVariable if it succeeds or None
 
         Examples
         --------
@@ -335,17 +386,21 @@ class AMDA_Webservice:
 
 
         """
-        # get list of parameters for this dataset
+        ds_range = self.dataset_range(dataset_id)
+        if not ds_range.intersect(DateTimeRange(start, stop)):
+            log.warning(f"You are requesting {dataset_id} outside of its definition range {ds_range}")
+            return None
+
         dataset_id = to_xmlid(dataset_id)
-        name = flat_inventories.amda.datasets[dataset_id].name
-        meta = {k: v for k, v in flat_inventories.amda.datasets[dataset_id].__dict__.items() if
-                not isinstance(v, AMDAIndex)}
+        name = self.flat_inventory.datasets[dataset_id].name
+        meta = {k: v for k, v in self.flat_inventory.datasets[dataset_id].__dict__.items() if
+                not isinstance(v, SpeasyIndex)}
         parameters = self.list_parameters(dataset_id)
         return Dataset(name=name, variables={p.name: self.get_parameter(p, start, stop, **kwargs) for p in parameters},
                        meta=meta)
 
-    @CacheCall(cache_retention=float(amda_user_cache_retention.get()))
-    def get_timetable(self, timetable_id: str or AMDATimetableIndex, **kwargs) -> Optional[TimeTable]:
+    @CacheCall(cache_retention=amda_cfg.user_cache_retention())
+    def get_timetable(self, timetable_id: str or TimetableIndex, **kwargs) -> Optional[TimeTable]:
         """Get timetable data by ID.
 
         Parameters
@@ -368,8 +423,8 @@ class AMDA_Webservice:
         """
         return self._impl.dl_timetable(to_xmlid(timetable_id), **kwargs)
 
-    @CacheCall(cache_retention=float(amda_user_cache_retention.get()))
-    def get_catalog(self, catalog_id: str or AMDACatalogIndex, **kwargs) -> Optional[Catalog]:
+    @CacheCall(cache_retention=amda_cfg.user_cache_retention())
+    def get_catalog(self, catalog_id: str or CatalogIndex, **kwargs) -> Optional[Catalog]:
         """Get catalog data by ID.
 
         Parameters
@@ -386,47 +441,13 @@ class AMDA_Webservice:
         --------
 
         >>> import speasy as spz
-        >>> spz.amda.get_catalog("sharedcatalog_0")
-        <Catalog: choc_MPB_catalogue_MEX>
+        >>> spz.amda.get_catalog("sharedcatalog_22")
+        <Catalog: model_regions_plasmas_mms_2019>
 
         """
         return self._impl.dl_catalog(to_xmlid(catalog_id), **kwargs)
 
-    def parameter_range(self, parameter_id: str or AMDAParameterIndex or AMDADatasetIndex) -> Optional[DateTimeRange]:
-        """Get product time range.
-
-        Parameters
-        ----------
-        parameter_id: str or AMDAParameterIndex or AMDADatasetIndex
-            parameter id
-
-        Returns
-        -------
-        Optional[DateTimeRange]
-            Data time range
-
-        Examples
-        --------
-
-        >>> import speasy as spz
-        >>> spz.amda.parameter_range("imf")
-        <DateTimeRange: 1997-09-02T00:00:12+00:00 -> ...>
-
-        """
-        if not len(flat_inventories.amda.parameters):
-            self._impl.update_inventory()
-        parameter_id = to_xmlid(parameter_id)
-        dataset_name = self._find_parent_dataset(parameter_id)
-
-        if dataset_name in flat_inventories.amda.datasets:
-            dataset = flat_inventories.amda.datasets[dataset_name]
-            return DateTimeRange(
-                datetime.strptime(dataset.dataStart, '%Y-%m-%dT%H:%M:%SZ'),
-                datetime.strptime(dataset.dataStop, '%Y-%m-%dT%H:%M:%SZ')
-            )
-
-    @staticmethod
-    def list_parameters(dataset_id: Optional[str or AMDADatasetIndex] = None) -> List[AMDAParameterIndex]:
+    def list_parameters(self, dataset_id: Optional[str or DatasetIndex] = None) -> List[ParameterIndex]:
 
         """Get the list of parameter indexes available in AMDA or a given dataset
 
@@ -448,17 +469,16 @@ class AMDA_Webservice:
         >>> len(amda_parameters) > 0
         True
         >>> amda_parameters[0]
-        <ParameterIndex: |b|>
+        <ParameterIndex: ...>
 
 
         """
 
         if dataset_id is not None:
-            return list(flat_inventories.amda.datasets[to_xmlid(dataset_id)])
-        return list(filter(is_public, flat_inventories.amda.parameters.values()))
+            return list(self.flat_inventory.datasets[to_xmlid(dataset_id)])
+        return list(filter(is_public, self.flat_inventory.parameters.values()))
 
-    @staticmethod
-    def list_catalogs() -> List[AMDACatalogIndex]:
+    def list_catalogs(self) -> List[CatalogIndex]:
         """Get the list of public catalog IDs:
 
         Returns
@@ -477,10 +497,9 @@ class AMDA_Webservice:
         <CatalogIndex: model_regions_plasmas_mms_2019>
 
         """
-        return list(filter(is_public, flat_inventories.amda.catalogs.values()))
+        return list(filter(is_public, self.flat_inventory.catalogs.values()))
 
-    @staticmethod
-    def list_user_timetables() -> List[AMDATimetableIndex]:
+    def list_user_timetables(self) -> List[TimetableIndex]:
         """Get the list of user timetables. User timetable are represented as dictionary objects.
 
         Returns
@@ -504,10 +523,9 @@ class AMDA_Webservice:
 
         """
         # get list of private parameters
-        return list(filter(is_private, flat_inventories.amda.timetables.values()))
+        return list(filter(is_private, self.flat_inventory.timetables.values()))
 
-    @staticmethod
-    def list_user_catalogs() -> List[AMDACatalogIndex]:
+    def list_user_catalogs(self) -> List[CatalogIndex]:
         """Get the list of user catalogs. User catalogs are represented as dictionary objects.
 
         Returns
@@ -531,10 +549,9 @@ class AMDA_Webservice:
 
         """
         # get list of private parameters
-        return list(filter(is_private, flat_inventories.amda.catalogs.values()))
+        return list(filter(is_private, self.flat_inventory.catalogs.values()))
 
-    @staticmethod
-    def list_user_parameters() -> List[AMDAParameterIndex]:
+    def list_user_parameters(self) -> List[ParameterIndex]:
         """Get the list of user parameters. User parameters are represented as dictionary objects.
 
         Returns
@@ -558,10 +575,9 @@ class AMDA_Webservice:
 
         """
         # get list of private parameters
-        return list(filter(is_private, flat_inventories.amda.parameters.values()))
+        return list(filter(is_private, self.flat_inventory.parameters.values()))
 
-    @staticmethod
-    def list_timetables() -> List[AMDATimetableIndex]:
+    def list_timetables(self) -> List[TimetableIndex]:
         """Get list of public timetables.
 
         Returns
@@ -574,13 +590,12 @@ class AMDA_Webservice:
 
         >>> import speasy as spz
         >>> spz.amda.list_timetables()[::50]
-        [<TimetableIndex: FTE_c1>, <TimetableIndex: Jian_SIR_list>, <TimetableIndex: MMS_Burst_Mode_2019January>]
+        [<TimetableIndex: ...>, <TimetableIndex: ...>, <TimetableIndex: ...>]
 
         """
-        return list(filter(is_public, flat_inventories.amda.timetables.values()))
+        return list(filter(is_public, self.flat_inventory.timetables.values()))
 
-    @staticmethod
-    def list_datasets() -> List[AMDADatasetIndex]:
+    def list_datasets(self) -> List[DatasetIndex]:
         """Get the list of dataset id available in AMDA_Webservice
 
         Returns
@@ -596,28 +611,26 @@ class AMDA_Webservice:
         >>> len(amda_datasets) > 0
         True
         >>> amda_datasets[0]
-        <DatasetIndex: final / prelim>
+        <DatasetIndex: ...>
         >>> amda_datasets[0].desc
-        'Interplanetary Magnetic Field 16-sec Level2/PRELIM Data<br/> Sampling: 16S<br/> Provider: CDAWeb'
+        '...'
 
         """
-        return list(filter(is_public, flat_inventories.amda.datasets.values()))
+        return list(filter(is_public, self.flat_inventory.datasets.values()))
 
-    @staticmethod
-    def _find_parent_dataset(product_id: str or AMDADatasetIndex or AMDAParameterIndex or AMDAComponentIndex) -> \
+    def _find_parent_dataset(self, product_id: str or DatasetIndex or ParameterIndex or ComponentIndex) -> \
         Optional[str]:
 
         product_id = to_xmlid(product_id)
-        product_type = AMDA_Webservice.product_type(product_id)
+        product_type = self.product_type(product_id)
         if product_type is ProductType.DATASET:
             return product_id
         elif product_type in (ProductType.COMPONENT, ProductType.PARAMETER):
-            for dataset in flat_inventories.amda.datasets.values():
+            for dataset in self.flat_inventory.datasets.values():
                 if product_id in dataset:
                     return to_xmlid(dataset)
 
-    @staticmethod
-    def product_type(product_id: str or AMDAIndex) -> ProductType:
+    def product_type(self, product_id: str or SpeasyIndex) -> ProductType:
         """Returns product type for any known ADMA product from its index or ID.
 
         Parameters
@@ -640,15 +653,15 @@ class AMDA_Webservice:
         <ProductType.DATASET: 1>
         """
         product_id = to_xmlid(product_id)
-        if product_id in flat_inventories.amda.datasets:
+        if product_id in self.flat_inventory.datasets:
             return ProductType.DATASET
-        if product_id in flat_inventories.amda.parameters:
+        if product_id in self.flat_inventory.parameters:
             return ProductType.PARAMETER
-        if product_id in flat_inventories.amda.components:
+        if product_id in self.flat_inventory.components:
             return ProductType.COMPONENT
-        if product_id in flat_inventories.amda.timetables:
+        if product_id in self.flat_inventory.timetables:
             return ProductType.TIMETABLE
-        if product_id in flat_inventories.amda.catalogs:
+        if product_id in self.flat_inventory.catalogs:
             return ProductType.CATALOG
 
         return ProductType.UNKNOWN
